import * as path from 'path';
import * as url from 'url';

import * as Electron from 'electron';

import log from 'electron-log';

import fs_backend from 'i18next-node-fs-backend';

import {
  AppChannel,
  WebContentsChannel,
  PlatformUtil,
  BrowserWindowChannel
} from '@ucap/electron-core';

import { ElectronLogAppender } from '@ucap/electron-logger';

import { AppApi, PlatformApi } from '@ucap/electron-common';

import { AppWindow } from './windows/app.window';

import { AppChatService } from './services/app-chat.service';
import { AppFileService } from './services/app-file.service';
import { AppIdleService } from './services/app-idle.service';
import { AppMessageService } from './services/app-message.service';
import { AppPlatformService } from './services/app-platform.service';
import { AppService } from './services/app.service';
import { AppLoggerService } from './services/app-logger.service';
import { AppI18nService } from './services/app-i18n.service';

Electron.app.disableHardwareAcceleration();
Electron.app.commandLine.appendSwitch('disable-software-rasterizer');

const rootPath = PlatformUtil.rootPath(__dirname).split(path.sep).join('/');
const distPath = path.join(rootPath, 'dist').split(path.sep).join('/');

const appIconPath = path
  .join(
    distPath,
    'assets/app/images/icon',
    __WIN32__ ? 'icon.ico' : __DARWIN__ ? 'icon.png' : 'icon.png'
  )
  .split(path.sep)
  .join('/');

export const SERVICES = [
  AppChatService,
  AppFileService,
  AppIdleService,
  AppMessageService,
  AppPlatformService,
  AppLoggerService,
  AppI18nService,
  AppService
];

@AppApi.AppSettings({
  bootstrap: AppWindow,
  services: [...SERVICES],

  assets: {
    appIcon: appIconPath
  },

  logger: {
    moduleConfig: {
      appenders: [new ElectronLogAppender()]
    }
  },

  i18n: {
    moduleConfig: {},
    i18next: {
      options: {
        whitelist: ['ko', 'en'],
        fallbackLng: 'en',
        debug: true,
        saveMissing: true,
        returnEmptyString: false,
        ns: ['electron'],
        backend: {
          loadPath: `${distPath}/assets/app/i18n/{{lng}}/{{ns}}.json`,
          addPath: `${distPath}/assets/app/i18n/{{lng}}/{{ns}}.missing.json`
        }
      },
      useBackends: [fs_backend]
    }
  },

  notification: {
    option: {
      width: 340,
      height: 100,
      padding: 0,
      borderRadius: 0,
      browserWindowPool: {
        min: 0,
        max: 7
      },
      displayTime: 5000,
      defaultStyleContainer: {},
      defaultStyleAppIcon: { display: 'none' },
      defaultStyleImage: {},
      defaultStyleClose: {},
      defaultStyleText: {}
    },
    preloadPath: path.join(distPath, 'assets/notification/preload.js'),
    templatePath: path.join(distPath, 'assets/notification/template.html'),
    defaultImagePath: path.join(
      distPath,
      'assets/notification/images/nophoto_50.png'
    ),
    defaultSoundPath: path.join(
      distPath,
      'assets/notification/sounds/message-alarm.mp3'
    )
  }
})
export class App implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'app';

  appLoggerService: AppLoggerService;
  appI18nService: AppI18nService;

  appChatService: AppChatService;
  appFileService: AppFileService;
  appIdleService: AppIdleService;
  appMessageService: AppMessageService;
  appPlatformService: AppPlatformService;
  appService: AppService;

  appWindow: AppWindow;

  constructor(private appConfiguration: AppApi.AppConfiguration) {}

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        const backends = this.appConfiguration.i18n.i18next.useBackends;
        if (!!backends && 0 < backends.length) {
          for (const backend of backends) {
            this.appI18nService.use(backend);
          }
        }

        await this.appI18nService.init(
          this.appConfiguration.i18n.i18next.options
        );
        await this.appI18nService.changeLanguage('ko');

        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @AppApi.On(AppChannel.secondInstance)
  onSecondInstance() {
    log.info('AppChannel.SecondInstance');
    if (!!this.appWindow && !!this.appWindow.native) {
      if (this.appWindow.native.isMinimized()) {
        this.appWindow.native.restore();
      }

      if (!this.appWindow.native.isVisible()) {
        this.appWindow.native.show();
      }

      this.appWindow.native.focus();
    }
  }

  @AppApi.On(AppChannel.ready)
  onReady() {
    log.info('AppChannel.Ready');

    this.appWindow = PlatformApi.Platform.buildBrowserWindow(AppWindow);

    if (__DEV__) {
      import('electron-debug').then((m) => {
        m.default({ showDevTools: true });
      });

      import('electron-devtools-installer').then((edi) => {
        const extensions = [edi.REDUX_DEVTOOLS];

        for (const extension of extensions) {
          try {
            edi.default(extension);
          } catch (e) {
            log.error(e);
          }
        }
      });
    }

    // log.debug('services', this.configuration.services);
    const nameOfAssets = '/assets/';

    Electron.protocol.interceptFileProtocol(
      'file',
      (request, callback) => {
        let requestUrl = decodeURIComponent(
          request.url.replace(/file:[/\\]*/, '')
        );

        const li = requestUrl.lastIndexOf('?');
        if (-1 !== li) {
          requestUrl = requestUrl.slice(0, li);
        }

        if (-1 !== requestUrl.indexOf(nameOfAssets)) {
          if (!requestUrl.startsWith(distPath)) {
            // log.info('requestUrl', requestUrl);
            const parts = requestUrl.split(nameOfAssets);
            if (2 !== parts.length) {
              log.error('file:// error', parts, requestUrl);
            } else {
              requestUrl = `${distPath}${nameOfAssets}${parts[1]}`;
            }
            // log.info('fixed', requestUrl);
          }
        }

        callback(requestUrl);
      },
      (error) => {
        log.error(error);
      }
    );

    this._loadURL();

    this.appWindow.native.on(BrowserWindowChannel.closed, () => {
      this.appWindow = null;
    });

    this.appWindow.native.webContents.on(
      WebContentsChannel.didFailLoad,
      (
        event: Event,
        errorCode: number,
        errorDescription: string,
        validatedURL: string,
        isMainFrame: boolean
      ) => {
        if ('ERR_FILE_NOT_FOUND' === errorDescription) {
          // log.warn('validatedURL', validatedURL);
          const parts = validatedURL.split('/#');
          if (2 === parts.length) {
            this.appService.setInitInfo({ initUrl: parts[1] });
          }

          this._loadURL();
          return;
        }
      }
    );
  }

  @AppApi.On(AppChannel.activate)
  onActivate() {}

  @AppApi.On(AppChannel.windowAllClosed)
  onWindowAllClosed() {
    // if ('darwin' !== process.platform) {
    //   Electron.app.quit();
    // }
  }

  private _loadURL() {
    this.appWindow.native.loadURL(
      __DEV__
        ? 'http://localhost:4200'
        : url.format({
            protocol: 'file:',
            slashes: true,
            pathname: path.resolve(distPath, 'index.html')
          })
    );
  }
}
